/*
  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.

  Copyright (c) 2003-2009 PRO ACTIVE SAS - www.proactive-group.com
   
  ref_desFire_pcsc.c
  ------------------

  This is the reference applications that validates the Pcsc_Desfire.dll
  with Philips DESFire cards.
  
  JDA 13/06/2008 : created from ref_desfire.c (CSB legacy SDK)

*/

#include <winscard.h>
#ifdef WIN32
#pragma comment(lib,"winscard.lib")
#endif

#include "cardware/desfire/pcsc_desfire.h"

#include <stdio.h>
#include <string.h>

#define PCSC_E_NO_READERS_AVAILABLE  0x8010002E

BOOL TestDesfire(SCARDCONTEXT hContext, const char *szReader);
BOOL TestDesfire_Ex(SCARDHANDLE hCard, BYTE level);

int main(int argc, char **argv)
{
  SCARDCONTEXT      hContext;
  SCARD_READERSTATE rgscState[MAXIMUM_SMARTCARD_READERS];

  DWORD             dwReaders, dwReadersOld = 0;

  LPSTR  szReaders = NULL;
  DWORD  dwReadersSz;

  BOOL   first_run = TRUE;
  DWORD  i, j;
  LONG   rc;

  /*
   * Welcome message, check parameters
   * ---------------------------------
   */
  printf("SpringCard SDK for PC/SC (CSB6 family)\n");
  printf("\n");
  printf("NXP DESFIRE EV1 reference demo\n");
  printf("------------------------------\n");
  printf("www.springcard.com\n\n");
  printf("\n");
  printf("Press <Ctrl+C> to exit\n\n");

  /*
   * Get a handle to the PC/SC resource manager
   * ------------------------------------------
   */
  rc = SCardEstablishContext(SCARD_SCOPE_SYSTEM, NULL, NULL, &hContext);
  if(rc != SCARD_S_SUCCESS)
  {
    printf("SCardEstablishContext :%X\n",rc);
	  return EXIT_FAILURE;
  }

  /*
   * Infinite loop, we'll exit only when killed by Ctrl+C
   * ----------------------------------------------------
   */
  for (;;)
  {
    /*
     * Get the list of available readers
  	 */

    dwReadersSz = SCARD_AUTOALLOCATE;
	  rc = SCardListReaders(hContext,
                            NULL,                /* Any group */
                            (LPTSTR) &szReaders, /* Diabolic cast for buffer auto-allocation */
                            &dwReadersSz);       /* Beg for auto-allocation */
	  
	if (rc == PCSC_E_NO_READERS_AVAILABLE)
	{
	  /* No reader at all */
	  dwReaders = 0;
	} else if (rc != SCARD_S_SUCCESS)
	{
      printf("SCardListReaders error %Xh\n",rc);
	  break;
	} else
	{
	  /* Track events on found readers. */
	  LPTSTR pReader = szReaders;

	  for (dwReaders=0; dwReaders<MAXIMUM_SMARTCARD_READERS; dwReaders++)
	  {
        /* End of multi-string array */
		if (*pReader == '\0') break;

        /* Remember this reader's name */
		rgscState[dwReaders].szReader = pReader;

        /* Jump to next entry in multi-string array */
		pReader += strlen(pReader) + 1;
	  }
	}

	if (first_run)
    {
      /* Program startup, display the number of readers */
      if (dwReaders == 0)
      {
        printf("No PC/SC reader\n\n");
      } else
      {
        printf("%d PC/SC reader%s found\n\n", dwReaders, dwReaders ? "s" : "");
      }
    }
      
	if (dwReadersOld != dwReaders)
	{
	  /* Reader added, or reader removed           */
      /* (re)set the initial state for all readers */
	  for (i=0; i<dwReaders; i++)
		rgscState[i].dwCurrentState = SCARD_STATE_UNAWARE;

      if (!first_run)
	  {
        int c;

	    /* Not the program startup, explain the event */
	    if (dwReadersOld > dwReaders)
		{
          c = (int) dwReadersOld-dwReaders;
		  printf("%d reader%s ha%s been removed from the system\n\n", c, (c==1)?" ":"s", (c==1)?"s":"ve");
		} else
		{
          c = (int) dwReaders-dwReadersOld;
		  printf("%d reader%s ha%s been added to the system\n\n", c, (c==1)?" ":"s", (c==1)?"s":"ve");
		}
	  }

      dwReadersOld = dwReaders;
	}

    if (dwReaders == 0)
    {
      /* We must wait here, because the SCardGetStatusChange doesn't wait  */
      /* at all in case there's no reader in the system. Silly, isn't it ? */
      Sleep(250);
    }

	/*
     * Interesting part of the job : call SCardGetStatusChange to monitor all changes
     * taking place in the PC/SC reader(s)
     */
	
	rc = SCardGetStatusChange(hContext, INFINITE, rgscState, dwReaders);	
		
	if (rc != SCARD_S_SUCCESS)
    {
      printf("SCardGetStatusChange error %Xh\n",rc);
      break;
    }

	for (i=0; i<dwReaders; i++)
	{
	  BOOL just_inserted = FALSE;

      if (rgscState[i].dwEventState & SCARD_STATE_CHANGED)
	  {
        /* Something changed since last loop        */
        if ( (rgscState[i].dwEventState & SCARD_STATE_PRESENT)
         && !(rgscState[i].dwCurrentState & SCARD_STATE_PRESENT) )
          just_inserted = TRUE;

        /* Remember new current state for next loop */
        rgscState[i].dwCurrentState = rgscState[i].dwEventState;
	  } else
	  {
        /* Nothing new, don't display anything for this reader */
        /* (unless we're in the first run)                     */
        if (!first_run)
          continue;
	  }

      /*
       * Display current reader's state
       * ------------------------------
       */

	  /* Reader's name */
      printf("Reader %d: %s\n", i, rgscState[i].szReader);

      /* Complete status */
      printf("\tCard state:\n");

      if (rgscState[i].dwEventState & SCARD_STATE_IGNORE)
        printf("\t\tIgnore this reader\n");

      if (rgscState[i].dwEventState & SCARD_STATE_UNKNOWN)
        printf("\t\tReader unknown\n");

      if (rgscState[i].dwEventState & SCARD_STATE_UNAVAILABLE)
        printf("\t\tStatus unavailable\n");

      if (rgscState[i].dwEventState & SCARD_STATE_EMPTY)
        printf("\t\tNo card in the reader\n");

      if (rgscState[i].dwEventState & SCARD_STATE_PRESENT)
        printf("\t\tCard present\n");

      if (rgscState[i].dwEventState & SCARD_STATE_ATRMATCH)
        printf("\t\tATR match\n");
  
      if (rgscState[i].dwEventState & SCARD_STATE_EXCLUSIVE)
        printf("\t\tReader reserved for exclusive use\n");

      if (rgscState[i].dwEventState & SCARD_STATE_INUSE)
        printf("\t\tReader/card in use\n");

      if (rgscState[i].dwEventState & SCARD_STATE_MUTE)
        printf("\t\tCard mute\n");

      if (just_inserted)
	  {
        /*
         * Display card's ATR
         * ------------------
         */

        if (rgscState[i].cbAtr)
		{
	      printf("\tCard ATR:");
		  for (j=0; j<rgscState[i].cbAtr; j++)
            printf(" %02X", rgscState[i].rgbAtr[j]);
          printf("\n");
		}

       /*
        * Do the Desfire library test on this card
        * ----------------------------------------
        */
        TestDesfire(hContext, rgscState[i].szReader);
	  }
	}

    /* Free the list of readers  */
    if (szReaders != NULL)
    {
      SCardFreeMemory(hContext, szReaders);
      szReaders = NULL;
    }
      
    /* Not the first run anymore */
		first_run = FALSE;
	}


  rc = SCardReleaseContext(hContext);

  /* Never go here (Ctrl+C kill us before !) */
  return EXIT_FAILURE;
}

BOOL TestDesfire(SCARDCONTEXT hContext, const char *szReader)
{
  SCARDHANDLE hCard;
  DWORD dwProtocol;
  LONG rc;
  BYTE b;
  BOOL f;

  printf("Testing the Desfire library on this card...\n");

  /*
   * Connect to the card, accept either T=0 or T=1
   * ---------------------------------------------
   */
  rc = SCardConnect(hContext,
                    szReader,
                    SCARD_SHARE_SHARED,
                    SCARD_PROTOCOL_T0|SCARD_PROTOCOL_T1,
                    &hCard,
                    &dwProtocol);
	if (rc != SCARD_S_SUCCESS)
  {
    printf("\tSCardConnect error %Xh\n",rc);
		return FALSE;
  }

  printf("\tConnected to the card, protocol ");
  switch (dwProtocol)
  {
    case SCARD_PROTOCOL_T0 : printf("T=0"); break;
    case SCARD_PROTOCOL_T1 : printf("T=1"); break;
    default                : printf("%08lXh", dwProtocol);
  }
  printf("\n");

  
  SCardDesfire_AttachLibrary(hCard);

//  SCardDesfire_IsoWrapping(hCard, DF_ISO_WRAPPING_READER);
//  SCardDesfire_IsoWrapping(hCard, DF_ISO_WRAPPING_CARD);

  f = TestDesfire_Ex(hCard, 2);
  /*
  for (b=1; b<=4; b++)
  {
    f = TestDesfire_Ex(hCard, b);
    if (!f) break;
  }*/
  SCardDesfire_DetachLibrary(hCard);

  SCardDisconnect(hCard, SCARD_EJECT_CARD);


  printf("\n\n\n");
  return f;
}
